@@ -49,7 +49,10 @@ impl<'a> Reader<'a> {
     }
 
     pub fn get(self, field: Field) -> Result<dynamic_value::Reader<'a>> {
-        assert_eq!(self.schema.raw, field.parent.raw);
+        assert!(core::ptr::eq(
+            self.schema.raw.generic,
+            field.parent.raw.generic
+        ));
         let ty = field.get_type();
         match field.get_proto().which()? {
             field::Slot(slot) => {
@@ -198,7 +201,10 @@ impl<'a> Reader<'a> {
     /// is active in the union and is not a null pointer. On non-union fields,
     /// returns `true` if the field is not a null pointer.
     pub fn has(&self, field: Field) -> Result<bool> {
-        assert_eq!(self.schema.raw, field.parent.raw);
+        assert!(core::ptr::eq(
+            self.schema.raw.generic,
+            field.parent.raw.generic
+        ));
         let proto = field.get_proto();
         if has_discriminant_value(proto) {
             let node::Struct(st) = self.schema.get_proto().which()? else {
@@ -240,7 +246,7 @@ impl<'a> Reader<'a> {
             Into::<crate::introspect::Type>::into(crate::introspect::TypeVariant::Struct(
                 self.schema.raw
             ))
-            .may_downcast_to(T::introspect())
+            .loose_equals(T::introspect())
         );
         self.reader.into()
     }
@@ -289,7 +295,10 @@ impl<'a> Builder<'a> {
     }
 
     pub fn get(self, field: Field) -> Result<dynamic_value::Builder<'a>> {
-        assert_eq!(self.schema.raw, field.parent.raw);
+        assert!(core::ptr::eq(
+            self.schema.raw.generic,
+            field.parent.raw.generic
+        ));
         let ty = field.get_type();
         match field.get_proto().which()? {
             field::Slot(slot) => {
@@ -446,7 +455,10 @@ impl<'a> Builder<'a> {
     }
 
     pub fn set(&mut self, field: Field, value: dynamic_value::Reader<'_>) -> Result<()> {
-        assert_eq!(self.schema.raw, field.parent.raw);
+        assert!(core::ptr::eq(
+            self.schema.raw.generic,
+            field.parent.raw.generic
+        ));
         self.set_in_union(field)?;
         let ty = field.get_type();
         match field.get_proto().which()? {
@@ -586,7 +598,10 @@ impl<'a> Builder<'a> {
     }
 
     pub fn init(mut self, field: Field) -> Result<dynamic_value::Builder<'a>> {
-        assert_eq!(self.schema.raw, field.parent.raw);
+        assert!(core::ptr::eq(
+            self.schema.raw.generic,
+            field.parent.raw.generic
+        ));
         self.set_in_union(field)?;
         let ty = field.get_type();
         match field.get_proto().which()? {
@@ -627,7 +642,10 @@ impl<'a> Builder<'a> {
     }
 
     pub fn initn(mut self, field: Field, size: u32) -> Result<dynamic_value::Builder<'a>> {
-        assert_eq!(self.schema.raw, field.parent.raw);
+        assert!(core::ptr::eq(
+            self.schema.raw.generic,
+            field.parent.raw.generic
+        ));
         self.set_in_union(field)?;
         let ty = field.get_type();
         match field.get_proto().which()? {
@@ -680,7 +698,10 @@ impl<'a> Builder<'a> {
     /// Clears a field, setting it to its default value. For pointer fields,
     /// this makes the field null.
     pub fn clear(&mut self, field: Field) -> Result<()> {
-        assert_eq!(self.schema.raw, field.parent.raw);
+        assert!(core::ptr::eq(
+            self.schema.raw.generic,
+            field.parent.raw.generic
+        ));
         self.set_in_union(field)?;
         let ty = field.get_type();
         match field.get_proto().which()? {
@@ -793,7 +814,7 @@ impl<'a> Builder<'a> {
             Into::<crate::introspect::Type>::into(crate::introspect::TypeVariant::Struct(
                 self.schema.raw
             ))
-            .may_downcast_to(T::introspect())
+            .loose_equals(T::introspect())
         );
         self.builder.into()
     }
@@ -809,7 +830,7 @@ impl<'a> crate::traits::SetterInput<crate::any_pointer::Owned> for Reader<'a> {
     }
 }
 
-impl<'a> core::fmt::Debug for Reader<'a> {
+impl core::fmt::Debug for Reader<'_> {
     fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         core::fmt::Debug::fmt(
             &::core::convert::Into::<crate::dynamic_value::Reader<'_>>::into(*self),
